#include "stdafx.h"
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h> // pro funkci exit()
#include "PocetDeti.h"


// a vytvorime globalni promennou typu ukazatel na Polozku. Ta nam bude ukazovat
// na prvni polozku v seznamu, zatim je seznam prazdny, takze ji inicializujeme
// hodnotou NULL
Polozka * Seznam = NULL;


// nyni muze byt main klidne na zacatku, i kdyz vola funkce ktere jsou napsane dole,
// protoze o nich vi uz z hlavickoveho souboru "PocetDeti.h"
int main(int argc, char* argv[])
{
	// zkontrolujeme parametry, ktere jsme dostali. Radky jsou spojeny ||, tj. NEBO,
	// takze kdyz bude platit aspon jeden z nich, je to spatne.
	if (argc != 3 // pokud nemame dva parametry + jmeno spustitelneho souboru
		|| strlen(argv[1]) != 2 // prvni parametr nema dva znaky
		|| argv[1][0] != '-' // prvni parametr nezacina pomlckou
		|| (argv[1][1] != 'j' && argv[1][1] != 'p')) // druhy znak prvniho parametru je jiny
		                                             // nez j nebo p
	{
		// vypiseme informaci a skoncime
		puts("Spatne parametry.");
		puts("PocetDeti.exe (-j|-p) <jmeno_souboru>");
		return -1;
	}

	// v druhem parametru je jmeno souboru, ten zkusime otevrit
	FILE * f = fopen(argv[2], "r");
	if (f == NULL)
	{
		// a kdyz se to nepovede, vypiseme hlasku a koncime
		printf("Chyba pri otevirani souboru %s.\n", argv[2]);
		return -1;
	}

	// soubor je otevreny, precteme ho a vytvorime setrideny spojacek
	// (to dela tato funkce)
	NactiSoubor(f, argv[1][1]);

	// mame nacteno, soubor uz neni potreba, zavreme ho
	fclose(f);
	
	// a nacteny spojacek vypiseme na obrazovku
	VypisSeznam();

	// vypsali jsme, budeme koncit, musime uvolnit naalokovanou pamet
	ZrusSeznam();

	getch();
	return 0;
}


// pomocna funkce, ktera nam bude porovnavat dve polozky. Polozky budeme chtit tridit,
// takze je potrebujeme porovnavat, ktera je vetsi a ktera je mensi. To ovsem zavisi
// na tom, jestli tridime podle jmena, nebo podle poctu deti. Kdyz tridime podle jmena,
// bude parametr trideni obsahovat znak 'j', pokud podle poctu, bude obsahovat znak 'p'.
// Funkce dostane dva ukazatele na dve polozky, ktere chceme porovnat, vrati 1 pokud
// leva polozka je mensi nez prava, jinak 0.
int JeMensi(Polozka * p1, Polozka * p2, char trideni)
{
	if (trideni == 'j' && strcmp(p1->Jmeno, p2->Jmeno) < 0)
		// tridime podle jmena a jmeno polozky p1 je ve slovniku drive (je mensi) nez
		// jmeno polozky p2 - vracime jednicku a koncime.
		return 1;

	else if (trideni == 'p' && p1->PocetDeti < p2->PocetDeti)
		// tridime podle poctu a pocet deti v p1 je mensi nez pocet deti v p2
		return 1;

	else
		// neplati nic z predchoziho, tj. v aktualnim trideni ('p' nebo 'j') je polozka
		// p1 vetsi nebo rovna polozce p2 - vracime nulu.
		return 0;
}


// pomocna funkce, ktera do seznamu zaradi polozku, na kterou dostane ukazatel
// (prvni parametr) a samozrejme aby ji mohla zaradit na spravne misto, musi dostat
// i informaci, podle ceho tridime (druhy parametr).
void ZaradNovouPolozku(Polozka * p, char trideni)
{
	// pro jistotu dame rovnou polozce ukazatel na dalsi NULL, kdyby se zaradila
	// na konec seznamu, tak aby v Dalsi bylo NULL, ktere rika, ze uz za ni nic neni
	p->Dalsi = NULL;

	// jestlize je Seznam == NULL, znamena to, ze v seznamu jeste neni ani jedna
	// polozka a ta prave vkladana bude prvni. Dame tedy do promenne Seznam ukazatel
	// na tuto pridavanou polozku a ukoncime funkci.
	if (Seznam == NULL)
	{
		Seznam = p;
		return;
	}

	// seznam neni prazdny, v seznamu neco je (jinak by se provedl return v predchozim
	// ifu). Pokud nase pridavana polozka je mensi nez prvni prvek v seznamu, musi prijit
	// na zacatek. Tuto situaci musime take osetrit specialne.
	if (JeMensi(p, Seznam, trideni))
	{
		p->Dalsi = Seznam; // ukazatel Dalsi v nasi polozce si bude ukazovat na to, co bylo
		                   // dosud prvni
		Seznam = p; // a globalni ukazatel na zacatek seznamu si ukaze na nasi polozku.
		return; // a koncime, polozka je zarazena
	}

	// nyni vime, ze v seznamu je aspon jedna polozka a nami zarazovana neprijde hned
	// na zacatek seznamu, prijde tedy nekam doprostred, nebo na konec.

	// Kdyz prochazime seznam, zjistime, ze nase polozka patri mezi dve jine, az kdyz
	// si prohledneme druhou z nich. Pro prepojeni pointeru tak, abychom nasi zaradili
	// mezi ne, musime ovsem mit pristup jeste i na tu predchazejici. Proto si jednak
	// pamatujeme aktualni kontrolovanou pozici, a jednak predchozi
	Polozka * aktualni = Seznam->Dalsi;
	Polozka * predchozi = Seznam;

	// dokud se nedostaneme na konec (aktualni neni NULL), prochazime seznam a kontrolujeme,
	// jestli nase pridavana polozka nepatri mezi predchozi a aktualni polozky.
	while (aktualni != NULL)
	{
		// kdyz bude nase pridavana mensi nez aktualni, ma se zaradit pred ni
		if (JeMensi(p, aktualni, trideni))
		{
			p->Dalsi = aktualni; // cili ukazatel "Dalsi" v pridavane si bude ukazovat na
			                     // aktualni
			predchozi->Dalsi = p; // predchozi si ukaze na pridavanou
			return; // a koncime, mame zarazeno
		}

		// pridavana neni mensi nez aktualni, tedy se posunujeme
		predchozi = aktualni; // ta co byla ted aktualni je predchozi
		aktualni = aktualni->Dalsi; // a aktualni je ta nasledujici
	}

	// while cyklus skoncil, nebyl prerusen returnem v ifu v nem, tedy jsme prosli cely
	// seznam a polozku jeste nezaradili. Zaradime ji tedy na konec - v promenne
	// predchozi je ukazatel na posledni prvek, jeho ukazovatko Dalsi tedy nastavime na
	// pridavanou polozku a tim ji zaradime na konec.
	// pridavana polozka jiz pointer Dalsi na NULL ma nastaveny - udelali jsme na zacatku
	// teto funkce
	predchozi->Dalsi = p;
}


// funkce, ktera dostane otevreny soubor a nacte ho do seznamu (a ten seznam bude jiz
// spravne setriden)
void NactiSoubor(FILE * f, char trideni)
{
	// pomocne pole znaku, do ktereho budeme postupne nacitat radky souboru
	char radka[200];

	// nacitme do te doby, nez bude konec souboru
	while (!feof(f))
	{
		// fgets precte radku. prvni parametr - zapise ji do pole radka, druhy parametr
		// rika, kolik maximalne znaku se ma precist, tj. velikost pole vydelena velikosti
		// jedne (treba prvni) pozice, to da pocet prvku v poli, cili pocet znaku, ktere
		// se do nej vejdou. A odecteme jednicku, aby se vesla ukoncovaci nula.
		// posledni parametr je, z jakeho souboru nacitame
		fgets(radka, sizeof(radka)/sizeof(radka[0]) - 1, f);

		// protoze fgets narozdil od gets do retezce zaradi i \n na konci radky, coz se nam
		// nelibi, najdeme tedy toto \n a prepiseme ho nulou, cimz ukoncime retezec o neco
		// driv
		char * c = radka; // ukazatel na prvni znak
		while (*c != '\n') c++; // posunujeme se dokud nenajdeme \n
		*c = '\0'; // a toto \n prepiseme \0

		// chceme pridat novou polozku, tedy ji potrebujeme nekam dat do pameti. Zde si
		// dynamicky rikame o novou pamet
		Polozka * nova_polozka = (Polozka*) malloc(sizeof(Polozka));
		if (nova_polozka == NULL)
		{
			// kdyz se nepovede naalokovat, vypiseme hlasku
			puts("Malo pameti!");
			exit(-1); // a funkce exit okamzite ukonci program, -1 je to, co se v tomto
			          // pripade ukonceni vraci z mainu.
		}

		// do nove naalokovane polozky napiseme pocet deti
		nova_polozka->PocetDeti = radka[0] - '0'; // vzdalenost znaku s cislem od znaku
		              // '0' v ASCII tabulce je hledany pocet
		              // zde vyuzivame, ze pocet deti je jen jednociferne cislo

		// do nove polozky nakopirujeme retezec od druhe pozice na radku (to je radka+2)
		// a kopirujeme maximalne pocet znaku, ktere se tam vejdou do pole
		strncpy(nova_polozka->Jmeno, radka+2, POCET_ZNAKU_JMENO);
		// a protoze strncpy(), jak se muzeme docist v napovede, neprida na konec
		// retezce koncovou nulu, pokud kopirovany retezec je vetsi nez cislo, ktere
		// dostane v poslednim parametru, pro jistotu na konec vzdycky tu nulu dame.
		nova_polozka->Jmeno[POCET_ZNAKU_JMENO] = '\0';

		// Tim jsme vytvorili novou polozku a zavolame pomocnou funkci, ktera ji zaradi
		// do seznamu
		ZaradNovouPolozku(nova_polozka, trideni);
	}
}


// pomocna funkce vypisujici seznam
void VypisSeznam()
{
	// ukazatel p bude postupne ukazovat na vsechny polozky v seznamu, tady mu
	// priradime adresu prvni polozky, kterou si pamatujeme v globalni promenne
	// Seznam
	Polozka * p = Seznam;

	// p se bude posouvat dal a dal v seznamu a dokud nebude NULL, budeme vypisovat
	while (p != NULL)
	{
		// vypiseme aktualni polozku
		printf("Jmeno: %s, pocet deti: %d\n", p->Jmeno, p->PocetDeti);
		// a posuneme se
		p = p->Dalsi;
	}
}


// protoze jsme dynamicky alokovali pamet pro polozky, musime vsechny na konci
// programu zase zrusit. Tato funkce postupne projde cely seznam a pamet vsech
// polozek uvolni.
void ZrusSeznam()
{
	// jako pri vypisovani, zacneme u prvni polozky a budeme se posouvat
	Polozka * p = Seznam;
	while (p != NULL)
	{
		// protoze aktualni polozku budeme rusit a mame v ni zaroven ulozen ukazatel
		// na dalsi polozku, musime se posunout driv, nez tu polozku zrusime. Zaroven
		// si ale musime zapamatovat ukazatel na aktualni rusenou, abychom ji vubec mohli
		// uvolnit.
		Polozka * ke_zruseni = p; // v promenne ke_zruseni si zapamtujeme ukazatel
		p = p->Dalsi; // posuneme se
		free(ke_zruseni); // a uvolnime
	}
}



